@@ -1,15 +1,14 @@
 #!/usr/bin/env python3
 # This file is part of Xpra.
-# Copyright (C) 2018-2023 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2018-2024 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import socket
-from typing import Dict
-from zeroconf import ServiceInfo, Zeroconf, __version__ as zeroconf_version #@UnresolvedImport
+from zeroconf import ServiceInfo, Zeroconf, __version__ as zeroconf_version
 
 from xpra.log import Logger
-from xpra.util import envbool, envint, first_time
+from xpra.util.env import envbool, first_time
 from xpra.net.net_util import get_interfaces_addresses
 from xpra.net.mdns import XPRA_TCP_MDNS_TYPE, XPRA_UDP_MDNS_TYPE
 from xpra.net.net_util import get_iface
@@ -19,31 +18,30 @@ log("python-zeroconf version %s", zeroconf_version)
 
 IPV6 = envbool("XPRA_ZEROCONF_IPV6", True)
 IPV6_LO = envbool("XPRA_ZEROCONF_IPV6_LOOPBACK", False)
-IPV4_LO = envint("XPRA_ZEROCONF_IPV4_LOOPBACK", 1)
-
 
 LOOPBACK_AFAM = {
-    "0.0.0.0"   : socket.AF_INET,
-    "::"        : socket.AF_INET6,
-    }
+    "0.0.0.0": socket.AF_INET,
+    "::": socket.AF_INET6,
+}
 
 
 def get_interface_index(host):
-    #we don't use interface numbers with zeroconf,
-    #so just return the interface name,
-    #which is also unique
+    # we don't use interface numbers with zeroconf,
+    # so just return the interface name,
+    # which is also unique
     return get_iface(host)
 
+
 def inet_ton(af, addr):
-    if af==socket.AF_INET:
+    if af == socket.AF_INET:
         return socket.inet_aton(addr)
-    return socket.inet_pton(af, addr)   #@UndefinedVariable
+    return socket.inet_pton(af, addr)  # @UndefinedVariable
 
 
-def txt_rec(text_dict) -> Dict:
-    #prevent zeroconf from mangling our ints into booleans:
+def txt_rec(text_dict) -> dict:
+    # prevent zeroconf from mangling our ints into booleans:
     new_dict = {}
-    for k,v in text_dict.items():
+    for k, v in text_dict.items():
         if isinstance(v, int):
             new_dict[k] = str(v)
         else:
@@ -55,13 +53,15 @@ class ZeroconfPublishers:
     """
     Expose services via python zeroconf
     """
-    def __init__(self, listen_on, service_name:str, service_type:str=XPRA_TCP_MDNS_TYPE, text_dict=None):
+
+    def __init__(self, listen_on, service_name: str, service_type: str = XPRA_TCP_MDNS_TYPE, text_dict=None):
         log("ZeroconfPublishers%s", (listen_on, service_name, service_type, text_dict))
-        self.services = []
-        self.ports = {}
-        def add_address(host, port, af=socket.AF_INET):
+        self.services: list[ZeroconfPublisher] = []
+        self.ports: dict[str, set[int]] = {}
+
+        def add_address(host: str, port: int, af=socket.AF_INET):
             try:
-                if af==socket.AF_INET6 and host.find("%"):
+                if af == socket.AF_INET6 and host.find("%"):
                     host = host.split("%")[0]
                 try:
                     host = socket.gethostbyname(host)
@@ -77,7 +77,7 @@ class ZeroconfPublishers:
             ports = set(self.ports.get(service_name, ()))
             log("add_address(%s, %s, %s) ports=%s", host, port, af, ports)
             ports.add(port)
-            if len(ports)>1:
+            if len(ports) > 1:
                 sn += f"-{len(ports)}"
             try:
                 zp = ZeroconfPublisher(address, host, port, sn, service_type, text_dict)
@@ -87,45 +87,32 @@ class ZeroconfPublishers:
             else:
                 self.services.append(zp)
                 self.ports[service_name] = ports
+
         for host, port in listen_on:
             if host.startswith("[") and host.endswith("]"):
                 host = host[1:-1]
             if host in ("0.0.0.0", "::"):
                 af = LOOPBACK_AFAM.get(host)
-                if af==socket.AF_INET6 and not IPV6_LO:
+                if af == socket.AF_INET6 and not IPV6_LO:
                     if first_time(f"zeroconf-{host}"):
                         log.info(f"python-zeroconf: {host!r} IPv6 loopback address is not supported")
                         log("try XPRA_ZEROCONF_IPV6_LOOPBACK=1 to enable it at your own risk")
-                    #means that IPV6 is False and "::" is not supported
-                    #at time of writing, https://pypi.org/project/zeroconf/ says:
-                    #_Listening on localhost (::1) does not work. Help with understanding why is appreciated._
+                    # means that IPV6 is False and "::" is not supported
+                    # at time of writing, https://pypi.org/project/zeroconf/ says:
+                    # _listening on localhost (::1) does not work. Help with understanding why is appreciated._
                     continue
-
                 # annoying: we have to enumerate all interfaces
-                iaddr = get_interfaces_addresses()
-
-                def add_iface(iface: str, addresses: dict) -> None:
-                    log("%r %s: %s", iface, af, addresses.get(af, {}))
+                for iface, addresses in get_interfaces_addresses().items():
+                    log("%s %s: %s", iface, af, addresses.get(af, {}))
                     for defs in addresses.get(af, ()):
                         addr = defs.get("addr")
                         if addr:
                             add_address(addr, port, af)
-
-                log(f"interface addresses: {iaddr!r}")
-                # ensure loopback is done last, as it may conflict:
-                loopback = iaddr.pop("lo", {})
-                for iface, addresses in iaddr.items():
-                    add_iface(iface, addresses)
-                # with IPV4_LO=2, always publish loopback,
-                # with IPV4_LO=1, only publish it if we don't have other addresses:
-                if loopback and (IPV4_LO == 2) or (IPV4_LO == 1 and not iaddr):
-                    add_iface("lo", loopback)
                 continue
-
-            if host=="":
+            if host == "":
                 host = "127.0.0.1"
             af = socket.AF_INET
-            if host.find(":")>=0:
+            if host.find(":") >= 0:
                 if IPV6:
                     af = socket.AF_INET6
                 else:
@@ -142,11 +129,16 @@ class ZeroconfPublishers:
 
     def update_txt(self, txt) -> None:
         for s in self.services:
-            s.update_txt(txt)
+            try:
+                s.update_txt(txt)
+            except TimeoutError:
+                log(f"update_txt({txt})", exc_info=True)
 
 
 class ZeroconfPublisher:
-    def __init__(self, address, host, port, service_name, service_type=XPRA_TCP_MDNS_TYPE, text_dict=None):
+    def __init__(self, address, host: str, port: int,
+                 service_name: str, service_type=XPRA_TCP_MDNS_TYPE,
+                 text_dict=None):
         log("ZeroconfPublisher%s", (address, host, port, service_name, service_type, text_dict))
         self.address = address
         self.host = host
@@ -156,27 +148,27 @@ class ZeroconfPublisher:
         self.kwargs = {}
         self.registered = False
         try:
-            #ie: service_name = localhost.localdomain :2 (ssl)
+            # ie: service_name = localhost.localdomain :2 (ssl)
             parts = service_name.split(" ", 1)
             regname = parts[0].split(".")[0]
-            if len(parts)==2:
+            if len(parts) == 2:
                 regname += parts[1]
             regname = regname.replace(":", "-")
             regname = regname.replace(" ", "-")
             regname = regname.replace("(", "")
             regname = regname.replace(")", "")
-            #ie: regname = localhost-2-ssl
-            st = service_type+"local."
-            regname += "."+st
+            # ie: regname = localhost-2-ssl
+            st = service_type + "local."
+            regname += "." + st
             td = txt_rec(text_dict or {})
             self.kwargs = {
-                "type_"         : st,       #_xpra._tcp.local.
-                "name"          : regname,
-                "server"        : regname,
-                "port"          : port,
-                "properties"    : td,
-                "addresses"     : [self.address],
-                }
+                "type_": st,  # "_xpra._tcp.local."
+                "name": regname,
+                "server": regname,
+                "port": port,
+                "properties": td,
+                "addresses": [self.address],
+            }
             service = ServiceInfo(**self.kwargs)
             log("ServiceInfo(%s)=%s", self.kwargs, service)
             self.service = service
@@ -216,7 +208,7 @@ class ZeroconfPublisher:
             self.zeroconf.update_service(si)
             self.service = si
         except KeyError as e:
-            #probably a race condition with cleanup
+            # probably a race condition with cleanup
             log("update_txt(%s)", txt, exc_info=True)
             log.warn("Warning: failed to update service")
             log.warn(" %s", e)
@@ -226,20 +218,25 @@ class ZeroconfPublisher:
 
 def main():
     import random
-    port = int(20000*random.random())+10000
-    #host = "127.0.0.1"
+    port = int(20000 * random.random()) + 10000
+    # host = "127.0.0.1"
     host = "0.0.0.0"
     host_ports = [(host, port)]
-    service_name = "test %s" % int(random.random()*100000)
-    from gi.repository import GLib  # @UnresolvedImport
+    service_name = "test %s" % int(random.random() * 100000)
+    from xpra.os_util import gi_import
+    GLib = gi_import("GLib")
     publishers = []
+
     def add(service_type):
-        publisher = ZeroconfPublishers(host_ports, service_name, service_type, {"somename":"somevalue"})
+        publisher = ZeroconfPublishers(host_ports, service_name, service_type, {"somename": "somevalue"})
         GLib.idle_add(publisher.start)
         publishers.append(publisher)
+
         def update_rec():
             publisher.update_txt({"somename": "someothervalue"})
-        GLib.timeout_add(10*1000, update_rec)
+
+        GLib.timeout_add(10 * 1000, update_rec)
+
     add(XPRA_TCP_MDNS_TYPE)
     add(XPRA_UDP_MDNS_TYPE)
     loop = GLib.MainLoop()
