@@ -1,58 +1,79 @@
 # This file is part of Xpra.
-# Copyright (C) 2015-2017 Antoine Martin <antoine@xpra.org>
+# Copyright (C) 2015-2023 Antoine Martin <antoine@xpra.org>
 # Xpra is released under the terms of the GNU GPL v2, or, at your option, any
 # later version. See the file COPYING for details.
 
 import os
 import sys
 import subprocess
+from queue import SimpleQueue, Empty
+from collections.abc import Callable
+from typing import Any
 
-from xpra.gtk_common.gobject_compat import import_glib, register_os_signals
-from xpra.util import repr_ellipsized, envint, envbool
+from xpra.gtk.signals import register_os_signals
+from xpra.util.str_fn import csv, repr_ellipsized, hexstr
+from xpra.util.env import envint, envbool, get_exec_env
 from xpra.net.bytestreams import TwoFileConnection
-from xpra.net.common import ConnectionClosedException
-from xpra.net.protocol import Protocol
-from xpra.os_util import Queue, setbinarymode, SIGNAMES, bytestostr, hexstr, WIN32, POSIX
-from xpra.child_reaper import getChildReaper
+from xpra.net.common import ConnectionClosedException, PACKET_TYPES, PacketType, PacketElement
+from xpra.net.protocol.socket_handler import SocketProtocol
+from xpra.net.protocol.constants import CONNECTION_LOST, GIBBERISH
+from xpra.common import noop
+from xpra.os_util import gi_import, WIN32
+from xpra.util.io import setbinarymode
+from xpra.util.system import SIGNAMES
+from xpra.util.child_reaper import getChildReaper
 from xpra.log import Logger
 
+GLib = gi_import("GLib")
+
 log = Logger("util")
 
-
-#this wrapper allows us to interact with a subprocess as if it was
-#a normal class with gobject signals
-#so that we can interact with it using a standard xpra protocol layer
-#there is a wrapper for the caller
-#and one for the class
-#they talk to each other through stdin / stdout,
-#using the protocol for encoding the data
+# this wrapper allows us to interact with a subprocess as if it was
+# a normal class with gobject signals
+# so that we can interact with it using a standard xpra protocol layer
+# there is a wrapper for the caller
+# and one for the class
+# they talk to each other through stdin / stdout,
+# using the protocol for encoding the data
 
 
 DEBUG_WRAPPER = envbool("XPRA_WRAPPER_DEBUG", False)
-#to make it possible to inspect files (more human readable):
+# to make it possible to inspect files (more human-readable):
 HEXLIFY_PACKETS = envbool("XPRA_HEXLIFY_PACKETS", False)
-#avoids showing a new console window on win32:
+# avoids showing a new console window on win32:
 WIN32_SHOWWINDOW = envbool("XPRA_WIN32_SHOWWINDOW", False)
+# assume that the subprocess is running the same version of xpra,
+# so we can set the packet aliases without exchanging and parsing caps:
+# (this can break, ie: if both python2 and python3 builds are installed
+# and the python2 builds are from an older version)
+LOCAL_ALIASES = envbool("XPRA_LOCAL_ALIASES", False)
+
+LOCAL_SEND_ALIASES = {v: i for i, v in enumerate(PACKET_TYPES)}
+LOCAL_RECEIVE_ALIASES = dict(enumerate(PACKET_TYPES))
+
+FLUSH = envbool("XPRA_SUBPROCESS_FLUSH", False)
 
 FAULT_RATE = envint("XPRA_WRAPPER_FAULT_INJECTION_RATE")
-def noop(_p):
-    pass
+
+
 INJECT_FAULT = noop
-if FAULT_RATE>0:
+if FAULT_RATE > 0:
     _counter = 0
-    def DO_INJECT_FAULT(p):
+
+    def do_inject_fault(p) -> None:
         global _counter
         _counter += 1
-        if (_counter % FAULT_RATE)==0:
+        if (_counter % FAULT_RATE) == 0:
             log.warn("injecting fault in %s", p)
             p.raw_write("junk", "Wrapper JUNK! added by fault injection code")
-    INJECT_FAULT = DO_INJECT_FAULT
+
+    INJECT_FAULT = do_inject_fault
 
 
-def setup_fastencoder_nocompression(protocol):
+def setup_fastencoder_nocompression(protocol) -> None:
     from xpra.net.packet_encoding import get_enabled_encoders, PERFORMANCE_ORDER
     encoders = get_enabled_encoders(PERFORMANCE_ORDER)
-    assert len(encoders)>0, "no packet encoders available!?"
+    assert len(encoders) > 0, "no packet encoders available!?"
     for encoder in encoders:
         try:
             protocol.enable_encoder(encoder)
@@ -60,11 +81,11 @@ def setup_fastencoder_nocompression(protocol):
             break
         except Exception as e:
             log("failed to enable %s: %s", encoder, e)
-    #we assume this is local, so no compression:
+    # we assume this is local, so no compression:
     protocol.enable_compressor("none")
 
 
-class subprocess_callee(object):
+class SubprocessCallee:
     """
     This is the callee side, wrapping the gobject we want to interact with.
     All the input received will be converted to method calls on the wrapped object.
@@ -73,6 +94,7 @@ class subprocess_callee(object):
     You can also call send() to pass packets back to the caller.
     (there is no validation of which signals are valid or not)
     """
+
     def __init__(self, input_filename="-", output_filename="-", wrapped_object=None, method_whitelist=None):
         self.name = ""
         self._input = None
@@ -81,22 +103,17 @@ class subprocess_callee(object):
         self.output_filename = output_filename
         self.method_whitelist = method_whitelist
         self.large_packets = []
-        #the gobject instance which is wrapped:
+        # the gobject instance which is wrapped:
         self.wrapped_object = wrapped_object
-        self.send_queue = Queue()
+        self.send_queue = SimpleQueue()
         self.protocol = None
-        register_os_signals(self.handle_signal)
+        register_os_signals(self.handle_signal, self.name)
         self.setup_mainloop()
 
-    def setup_mainloop(self):
-        glib = import_glib()
-        self.mainloop = glib.MainLoop()
-        self.idle_add = glib.idle_add
-        self.timeout_add = glib.timeout_add
-        self.source_remove = glib.source_remove
+    def setup_mainloop(self) -> None:
+        self.mainloop = GLib.MainLoop()
 
-
-    def connect_export(self, signal_name, *user_data):
+    def connect_export(self, signal_name: str, *user_data) -> None:
         """ gobject style signal registration for the wrapped object,
             the signals will automatically be forwarded to the wrapper process
             using send(signal_name, *signal_args, *user_data)
@@ -105,14 +122,13 @@ class subprocess_callee(object):
         args = list(user_data) + [signal_name]
         self.wrapped_object.connect(signal_name, self.export, *args)
 
-    def export(self, *args):
+    def export(self, *args) -> None:
         signal_name = args[-1]
         log("export(%s, ...)", signal_name)
         data = args[1:-1]
         self.send(signal_name, *tuple(data))
 
-
-    def start(self):
+    def start(self) -> int:
         self.protocol = self.make_protocol()
         self.protocol.start()
         try:
@@ -123,8 +139,8 @@ class subprocess_callee(object):
             if str(e):
                 log.warn("%s", e)
             return 0
-        except Exception:
-            log.error("error in main loop", exc_info=True)
+        except RuntimeError:
+            log.error("Error in subprocess main loop", exc_info=True)
             return 1
         finally:
             log("run() ended, calling cleanup and protocol close")
@@ -137,89 +153,88 @@ class subprocess_callee(object):
                 self._input = None
                 try:
                     i.close()
-                except (OSError, IOError):
+                except OSError:
                     log("%s.close()", i, exc_info=True)
             o = self._output
             if o:
                 self._output = None
                 try:
                     o.close()
-                except (OSError, IOError):
+                except OSError:
                     log("%s.close()", o, exc_info=True)
 
-    def make_protocol(self):
-        #figure out where we read from and write to:
-        if self.input_filename=="-":
-            #disable stdin buffering:
-            self._input = os.fdopen(sys.stdin.fileno(), 'rb', 0)
+    def make_protocol(self) -> SocketProtocol:
+        # figure out where we read from and write to:
+        if self.input_filename == "-":
+            # disable stdin buffering:
+            self._input = os.fdopen(sys.stdin.fileno(), "rb", 0)
             setbinarymode(self._input.fileno())
         else:
-            self._input = open(self.input_filename, 'rb')
-        if self.output_filename=="-":
-            #disable stdout buffering:
-            self._output = os.fdopen(sys.stdout.fileno(), 'wb', 0)
+            self._input = open(self.input_filename, "rb")
+        if self.output_filename == "-":
+            # disable stdout buffering:
+            self._output = os.fdopen(sys.stdout.fileno(), "wb", 0)
             setbinarymode(self._output.fileno())
         else:
-            self._output = open(self.output_filename, 'wb')
-        #stdin and stdout wrapper:
+            self._output = open(self.output_filename, "wb")
+        # stdin and stdout wrapper:
         conn = TwoFileConnection(self._output, self._input,
                                  abort_test=None, target=self.name,
                                  socktype=self.name, close_cb=self.net_stop)
         conn.timeout = 0
-        protocol = Protocol(self, conn, self.process_packet, get_packet_cb=self.get_packet)
+        protocol = SocketProtocol(conn, self.process_packet, get_packet_cb=self.get_packet)
+        if LOCAL_ALIASES:
+            protocol.send_aliases = LOCAL_SEND_ALIASES
+            protocol.receive_aliases = LOCAL_RECEIVE_ALIASES
         setup_fastencoder_nocompression(protocol)
         protocol.large_packets = self.large_packets
         return protocol
 
-
-    def run(self):
+    def run(self) -> None:
         self.mainloop.run()
 
-
-    def net_stop(self):
-        #this is called from the network thread,
-        #we use idle add to ensure we clean things up from the main thread
+    def net_stop(self) -> None:
+        # this is called from the network thread,
+        # we use idle add to ensure we clean things up from the main thread
         log("net_stop() will call stop from main thread")
-        self.idle_add(self.stop)
+        GLib.idle_add(self.stop)
 
+    def cleanup(self) -> None:
+        """ subclasses may override this method """
 
-    def cleanup(self):
-        pass
-
-    def stop(self):
-        self.cleanup()
+    def stop(self) -> None:
         p = self.protocol
         log("stop() protocol=%s", p)
+        self.cleanup()
         if p:
             self.protocol = None
             p.close()
         self.do_stop()
 
-    def do_stop(self):
-        log("stop() stopping mainloop %s", self.mainloop)
+    def do_stop(self) -> None:
+        log("do_stop() stopping mainloop %s", self.mainloop)
         self.mainloop.quit()
 
-    def handle_signal(self, sig):
+    def handle_signal(self, sig) -> None:
         """ This is for OS signals SIGINT and SIGTERM """
-        #next time, just stop:
-        register_os_signals(self.signal_stop)
+        # next time, just stop:
+        register_os_signals(self.signal_stop, self.name)
         signame = SIGNAMES.get(sig, sig)
         log("handle_signal(%s) calling stop from main thread", signame)
         self.send("signal", signame)
-        self.timeout_add(0, self.cleanup)
-        #give time for the network layer to send the signal message
-        self.timeout_add(150, self.stop)
+        GLib.timeout_add(0, self.cleanup)
+        # give time for the network layer to send the signal message
+        GLib.timeout_add(150, self.stop)
 
-    def signal_stop(self, sig):
+    def signal_stop(self, sig) -> None:
         """ This time we really want to exit without waiting """
         signame = SIGNAMES.get(sig, sig)
         log("signal_stop(%s) calling stop", signame)
         self.stop()
 
-
-    def send(self, *args):
+    def send(self, *args) -> None:
         if HEXLIFY_PACKETS:
-            args = args[:1]+[hexstr(str(x)[:32]) for x in args[1:]]
+            args = list(args[:1]) + [hexstr(str(x)[:32]) for x in args[1:]]
         log("send: adding '%s' message (%s items already in queue)", args[0], self.send_queue.qsize())
         self.send_queue.put(args)
         p = self.protocol
@@ -227,37 +242,38 @@ class subprocess_callee(object):
             p.source_has_more()
         INJECT_FAULT(p)
 
-    def get_packet(self):
+    def get_packet(self) -> tuple[PacketType, bool, bool]:
         try:
             item = self.send_queue.get(False)
-        except Exception:
+            more = self.send_queue.qsize() > 0
+        except Empty:
             item = None
-        return (item, None, None, self.send_queue.qsize()>0)
+            more = False
+        return item, False, more
 
-    def process_packet(self, proto, packet):
-        command = bytestostr(packet[0])
-        if command==Protocol.CONNECTION_LOST:
+    def process_packet(self, proto, packet: PacketType) -> None:
+        command = str(packet[0])
+        if command == CONNECTION_LOST:
             log("connection-lost: %s, calling stop", packet[1:])
             self.net_stop()
             return
-        if command==Protocol.GIBBERISH:
+        if command == GIBBERISH:
             log.warn("gibberish received:")
             log.warn(" %s", repr_ellipsized(packet[1], limit=80))
             log.warn(" stopping")
             self.net_stop()
             return
-        if command=="stop":
+        if command == "stop":
             log("received stop message")
             self.net_stop()
             return
-        if command=="exit":
+        if command == "exit":
             log("received exit message")
             sys.exit(0)
-            return
-        #make it easier to hookup signals to methods:
+        # make it easier to hookup signals to methods:
         attr = command.replace("-", "_")
         if self.method_whitelist is not None and attr not in self.method_whitelist:
-            log.warn("invalid command: %s (not in whitelist: %s)", attr, self.method_whitelist)
+            log.warn("invalid command %r, not in whitelist: %s", attr, csv(self.method_whitelist))
             return
         wo = self.wrapped_object
         if not wo:
@@ -270,46 +286,36 @@ class subprocess_callee(object):
             return
         if DEBUG_WRAPPER:
             log("calling %s.%s%s", wo, attr, str(tuple(packet[1:]))[:128])
-        self.idle_add(method, *packet[1:])
+        GLib.idle_add(method, *packet[1:])
         INJECT_FAULT(proto)
 
 
-def exec_kwargs():
-    kwargs = {}
-    stderr = sys.stderr.fileno()
-    if POSIX:
-        kwargs["close_fds"] = True
-    elif WIN32:
+def exec_kwargs(**kwargs) -> dict[str, Any]:
+    if WIN32:
         from xpra.platform.win32 import REDIRECT_OUTPUT
         if REDIRECT_OUTPUT:
-            #stderr is not valid and would give us this error:
+            # stderr is not valid and would give us this error:
             # WindowsError: [Errno 6] The handle is invalid
-            stderr = open(os.devnull, 'w')
+            if "stderr" not in kwargs:
+                kwargs["stderr"] = open(os.devnull, 'w')
         if not WIN32_SHOWWINDOW:
-            startupinfo = subprocess.STARTUPINFO()
-            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
-            startupinfo.wShowWindow = 0     #aka win32.con.SW_HIDE
+            startupinfo = subprocess.STARTUPINFO()  # @UndefinedVariable
+            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW  # @UndefinedVariable
+            startupinfo.wShowWindow = 0  # aka win32.con.SW_HIDE
             kwargs["startupinfo"] = startupinfo
-    kwargs["stderr"] = stderr
+    if "stderr" not in kwargs and sys.stderr:
+        kwargs["stderr"] = sys.stderr.fileno()
     return kwargs
 
-def exec_env(blacklist=("LS_COLORS", )):
-    env = os.environ.copy()
+
+def exec_env() -> dict[str, str]:
+    env = get_exec_env()
     env["XPRA_SKIP_UI"] = "1"
     env["XPRA_FORCE_COLOR_LOG"] = "1"
-    #let's make things more complicated than they should be:
-    #on win32, the environment can end up containing unicode, and subprocess chokes on it
-    for k,v in env.items():
-        if k in blacklist:
-            continue
-        try:
-            env[k] = bytestostr(v.encode("utf8"))
-        except Exception:
-            env[k] = bytestostr(v)
     return env
 
 
-class subprocess_caller(object):
+class SubprocessCaller:
     """
     This is the caller side, wrapping the subprocess.
     You can call send() to pass packets to it
@@ -324,83 +330,86 @@ class subprocess_caller(object):
         self.protocol = None
         self.command = None
         self.description = description
-        self.send_queue = Queue()
-        self.signal_callbacks = {}
+        self.send_queue: SimpleQueue[PacketType] = SimpleQueue()
+        self.signal_callbacks: dict[str, list[tuple[Callable, list[Any]]]] = {}
         self.large_packets = []
-        #hook a default packet handlers:
-        self.connect(Protocol.CONNECTION_LOST, self.connection_lost)
-        self.connect(Protocol.GIBBERISH, self.gibberish)
-        glib = import_glib()
-        self.idle_add = glib.idle_add
-        self.timeout_add = glib.timeout_add
-        self.source_remove = glib.source_remove
+        # hook a default packet handlers:
+        self.connect(CONNECTION_LOST, self.connection_lost)
+        self.connect(GIBBERISH, self.gibberish)
 
-
-    def connect(self, signal, cb, *args):
+    def connect(self, signal: str, cb: Callable, *args) -> None:
         """ gobject style signal registration """
         self.signal_callbacks.setdefault(signal, []).append((cb, list(args)))
 
-
-    def subprocess_exit(self, *args):
-        #beware: this may fire more than once!
+    def subprocess_exit(self, *args) -> None:
+        # beware: this may fire more than once!
         log("subprocess_exit%s command=%s", args, self.command)
         self._fire_callback("exit")
 
-    def start(self):
+    def start(self) -> None:
         assert self.process is None, "already started"
         self.process = self.exec_subprocess()
         self.protocol = self.make_protocol()
         self.protocol.start()
 
-    def abort_test(self, action):
+    def abort_test(self, action) -> None:
         p = self.process
         if p is None or p.poll():
-            raise ConnectionClosedException("cannot %s: subprocess has terminated" % action)
+            raise ConnectionClosedException("cannot %s: subprocess has terminated" % action) from None
 
-    def make_protocol(self):
-        #make a connection using the process stdin / stdout
+    def make_protocol(self) -> SocketProtocol:
+        # make a connection using the process stdin / stdout
         conn = TwoFileConnection(self.process.stdin, self.process.stdout,
                                  abort_test=self.abort_test, target=self.description,
                                  socktype=self.description, close_cb=self.subprocess_exit)
         conn.timeout = 0
-        protocol = Protocol(self, conn, self.process_packet, get_packet_cb=self.get_packet)
+        protocol = SocketProtocol(conn, self.process_packet, get_packet_cb=self.get_packet)
+        if LOCAL_ALIASES:
+            protocol._log_stats = envbool("XPRA_LOG_SOCKET_STATS", False)
+            protocol.send_aliases = LOCAL_SEND_ALIASES
+            protocol.receive_aliases = LOCAL_RECEIVE_ALIASES
         setup_fastencoder_nocompression(protocol)
         protocol.large_packets = self.large_packets
         return protocol
 
-
-    def exec_subprocess(self):
+    def exec_subprocess(self) -> subprocess.Popen:
         kwargs = exec_kwargs()
         env = self.get_env()
         log("exec_subprocess() command=%s, env=%s, kwargs=%s", self.command, env, kwargs)
-        proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, stdout=subprocess.PIPE, env=env, **kwargs)
+        proc = subprocess.Popen(self.command, stdin=subprocess.PIPE, stdout=subprocess.PIPE,
+                                env=env, start_new_session=True, **kwargs)
         getChildReaper().add_process(proc, self.description, self.command, True, True, callback=self.subprocess_exit)
         return proc
 
-    def get_env(self):
+    def get_env(self) -> dict[str, str]:
         env = exec_env()
         env["XPRA_LOG_PREFIX"] = "%s " % self.description
         env["XPRA_FIX_UNICODE_OUT"] = "0"
+        # MS Windows wrappers should never wait for input:
+        env["XPRA_WAIT_FOR_INPUT"] = "0"
         return env
 
-    def cleanup(self):
+    def cleanup(self) -> None:
         self.stop()
 
-    def stop(self):
+    def stop(self) -> None:
         self.stop_process()
         self.stop_protocol()
 
-    def stop_process(self):
+    def stop_process(self) -> None:
         log("%s.stop_process() sending stop request to %s", self, self.description)
         proc = self.process
-        if proc and proc.poll() is None:
+        if not proc:
+            return
+        self.process = None
+        if proc.poll() is None:
             try:
                 proc.terminate()
-                self.process = None
             except Exception as e:
-                log.warn("failed to stop the wrapped subprocess %s: %s", proc, e)
+                log("stop_process() proc=%s", proc, exc_info=True)
+                log.warn("Warning: failed to stop the wrapped subprocess %s: %s", proc, e)
 
-    def stop_protocol(self):
+    def stop_protocol(self) -> None:
         p = self.protocol
         if p:
             self.protocol = None
@@ -408,47 +417,49 @@ class subprocess_caller(object):
             try:
                 p.close()
             except Exception as e:
-                log.warn("failed to close the subprocess connection: %s", p, e)
+                log("stop_protocol() protocol=%s", p, exc_info=True)
+                log.warn("Warning: failed to close the subprocess connection %s: %s", p, e)
 
-
-    def connection_lost(self, *args):
+    def connection_lost(self, *args) -> None:
         log("connection_lost%s", args)
         self.stop()
 
-    def gibberish(self, *args):
+    def gibberish(self, *args) -> None:
         log.warn("%s stopping on gibberish:", self.description)
         log.warn(" %s", repr_ellipsized(args[1], limit=80))
         self.stop()
 
-
-    def get_packet(self):
+    def get_packet(self) -> tuple[PacketType, bool, bool]:
         try:
             item = self.send_queue.get(False)
-        except Exception:
+            more = self.send_queue.qsize() > 0
+        except Empty:
             item = None
-        return (item, None, None, None, False, self.send_queue.qsize()>0)
+            more = False
+        return item, False, more
 
-    def send(self, *packet_data):
+    def send(self, *packet_data: PacketElement) -> None:
         self.send_queue.put(packet_data)
         p = self.protocol
         if p:
             p.source_has_more()
+            if FLUSH:
+                conn = p._conn
+                if conn and conn.is_active():
+                    conn.flush()
         INJECT_FAULT(p)
 
-    def process_packet(self, proto, packet):
+    def process_packet(self, proto, packet: PacketType) -> None:
         if DEBUG_WRAPPER:
             log("process_packet(%s, %s)", proto, [str(x)[:32] for x in packet])
-        signal_name = bytestostr(packet[0])
+        signal_name = str(packet[0])
         self._fire_callback(signal_name, packet[1:])
         INJECT_FAULT(proto)
 
-    def _fire_callback(self, signal_name, extra_args=()):
-        callbacks = self.signal_callbacks.get(signal_name)
+    def _fire_callback(self, signal_name: str, extra_args=()) -> None:
+        callbacks = self.signal_callbacks.get(signal_name, [])
         log("firing callback for '%s': %s", signal_name, callbacks)
-        if callbacks:
-            for cb, args in callbacks:
-                try:
-                    all_args = list(args) + list(extra_args)
-                    self.idle_add(cb, self, *all_args)
-                except Exception:
-                    log.error("error processing callback %s for %s packet", cb, signal_name, exc_info=True)
+        for cb, args in callbacks:
+            with log.trap_error(f"Error processing callback {cb} for {signal_name} packet"):
+                all_args = list(args) + list(extra_args)
+                GLib.idle_add(cb, self, *all_args)
